#include <stdlib.h>

#include <firestorm.h>
#include <packet.h>
#include <alert.h>
#include <signature.h>
#include <matcher.h>
#include <plugin.h>
#include <decode.h>
#include <args.h>
#include <detect.h>
#include <netinet/in.h>
#include <stdio.h>
#include <string.h>


PLUGIN_STD_DEFS();

proc_dispatch dispatch;
struct alert *cur_alert;
unsigned int alert_depth;
unsigned int cur_depth;
struct sig_node ipx_root;

struct generator ipx_gen=init_generator("sig.ipx", NULL);
	

proc_matcher_find matcher_find;
proc_args_parse args_parse;
proc_generator_add generator_add;

/* Detection stuff */
proc_detect_add_sig detect_add_sig;
proc_detect_free_sig detect_free_sig;
proc_detect detect;
proc_detect_optimize detect_optimize;
proc_alert alert;
proc_dispatch dispatch;

struct alert *cur_alert;
unsigned int alert_depth;
unsigned int cur_depth;

/* IPX: Internetwork Packet eXchange */
int ipx_print(struct layer *l, char *buf, int buflen);
void ipx_decode(struct packet *);
void ipx_match(struct packet *p, unsigned int l);
int ipx_commit(struct rule *r);
void ipx_optimize(void);
struct proto ipx_p={
	.name="ipx",
	.decode=ipx_decode,
	.print=ipx_print,
	.sig_add=ipx_commit,
	.sig_match=ipx_match,
	.sig_optimize=ipx_optimize,
};
struct proto_req ipx_r[]={
        proto_request("ethernet", __constant_htons(0x8137)),
        proto_request("sll", __constant_htons(0x8137)),
        proto_request("linux", __constant_htons(0x8137)),
        proto_request("802.3-novell", 0),
        proto_request("llc", 0xe0),
	null_request()
};

char *pnames[]={
	"SAP/NSP/Hello", "RIP", "Echo", "Error",
	"PEP", "SPX",
};
int ipx_print(struct layer *l, char *buf, int buflen)
{
	char *ptype="Unknown";
	char *b=buf;
	int c;
	char *x;

	switch (l->h.ipx->type) {
		case 0 ... 5:
			ptype=pnames[l->h.ipx->type];
			break;
		case 17:
			ptype="NCP";
			break;
		case 20:
			ptype="NetBIOS";
			break;
		case 104:
			ptype="NDS NCP";
			break;
	};
	x=l->h.ipx->src.node;
	for(c=0; c<6; c++, x++)
		b+=sprintf(b, "%02x%c", *x&0xFF, c==5 ? ' ' : ':');
	b+=sprintf(b,"> ");
	x=l->h.ipx->dst.node;
	for(c=0; c<6; c++, x++)
		b+=sprintf(b, "%02x%c", *x&0xFF, c==5 ? ' ' : ':');
	b+=sprintf(b,"type=0x%.2x(%s) net=0x%.8x:0x%.8x skt=0x%.4x:0x%.4x ",
				l->h.ipx->type,
				ptype,
				ntohl(l->h.ipx->src.net),
				ntohl(l->h.ipx->dst.net),
				ntohs(l->h.ipx->src.sock),
				ntohs(l->h.ipx->dst.sock));
	return 0;
}

void ipx_decode(struct packet *p)
{
	struct proto_child *pc;
	struct layer *l=&p->layer[p->llen];

	if ( (p->layer[p->llen+1].h.raw=
		l->h.raw+sizeof(struct pkt_ipxhdr))
		> p->end ) return;

	p->llen++;

	if ( p->llen >= PKT_LAYERS ) return;

	for(pc=l->proto->children; pc; pc=pc->next)
	{
		if ( l->h.ipx->dst.sock == pc->id ) {
			p->layer[p->llen].flags=0;
			p->layer[p->llen].session=NULL;
			p->layer[p->llen].proto=pc->proto;
			pc->proto->decode(p);
			return;
		}
	}

	if ( p->layer[p->llen].h.raw<p->end )
		p->layer[p->llen++].proto=NULL;
	dispatch(p);
}

void ipx_optimize(void)
{
	detect_optimize(&ipx_root);
}

void ipx_match(struct packet *p, unsigned int l)
{
	detect(&ipx_root, p, l);
}

int ipx_commit(struct rule *r)
{
	struct criteria *c;
	struct sig_node *x;
	struct matcher *m;
	struct alert *a;
	unsigned int i;
	
	/* Can't handle this case */
	if ( r->num_criteria==0 ) 
		return 0;

	if ( !(x=calloc(r->num_criteria, sizeof(*x))) ) 
		return 0;

	/* Copy the alert data */
	if ( !(a=calloc(1, sizeof(*a))) ) {
		free(x);
		return 0;
	}
	memcpy(a, &r->alert, sizeof(*a));

	/* Build an array of all the nodes we want to add */
	for(c=r->criteria,i=0; c; c=c->next,i++) {
		char *key;

		if ( !strcmp(c->crit, "src") ) {
			key="ipx_src";
		}else if ( !strcmp(c->crit, "dst") ) {
			key="ipx_dst";	
		}else if ( !strcmp(c->crit, "dport") ) {
			key="ipx_dstsock";	
		}else if ( !strcmp(c->crit, "sport") ) {
			key="ipx_srcsock";
		}else key = c->crit;
                if ( !(m=matcher_find(key)) ) {
                        mesg(M_ERR,"ipx_commit: cannot find '%s' matcher", key);
			detect_free_sig(x, i);
			return 0;
		}

		x[i].match=m;
		x[i].n=c->negate;
		x[i].cost=m->cost;
		if ( !(x[i].m=m->validate(c->args,
			&x[i].p, c->modifier, &x[i].cost)) ) {
			detect_free_sig(x, i);
			return 0;
		}
	}
	return detect_add_sig(x, r->num_criteria, &ipx_root, a);
}

int PLUGIN_DECODE (struct decode_api *d)
{
	int ok=0;

	object_check(d);

	dispatch=d->dispatch;

	ok+=d->decode_add(&ipx_p, ipx_r);

	return (ok) ? PLUGIN_ERR_OK : PLUGIN_ERR_FAIL;
}

int PLUGIN_INIT (struct plugin_in *in, struct plugin_out *out)
{
	plugin_check(in, out);

	PLUGIN_ID("decode.ipx", "Internetwork Packet eXchange");
	PLUGIN_VERSION(1, 1);
	PLUGIN_AUTHOR("John Leach", "john@johnleach.co.uk");
	PLUGIN_LICENSE("GPL");

	if ( !(matcher_find=in->import("matcher.find")) ) {
		return PLUGIN_ERR_OBJECT;
	}

	if ( !(generator_add=in->import("generator.add")) ) {
		return PLUGIN_ERR_OBJECT;
	}
	
	if ( !(alert=in->import("alert")) ) {
		return PLUGIN_ERR_OBJECT;
	}

	if ( !(detect_add_sig=in->import("detect.add_sig")) ) {
		return PLUGIN_ERR_OBJECT;
	}

	if ( !(detect_free_sig=in->import("detect.free_sig")) ) {		
		return PLUGIN_ERR_OBJECT;
	}

	if ( !(detect=in->import("detect")) ) {
		return PLUGIN_ERR_OBJECT;
	}

	if ( !(detect_optimize=in->import("detect.optimize")) ) {
		return PLUGIN_ERR_OBJECT;
	}

	return PLUGIN_ERR_OK;
}

int PLUGIN_UNLOAD (int code) {
	return PLUGIN_ERR_OK;
}
