/*
 * This file is part of tcpidx
 * Copyright (c) 2003 Gianni Tedesco
 * Released under the terms of the GNU GPL version 2
 *
 * fbuf - Fast file buffering code with inlined fast path. Amortizes
 * that function call overhead if looping and writing lots of very
 * small amounts of data each time. Only works for write buffering.
 *
 * WARNING WARNING WARNING WARNING WARNING WARNING WARNING WARNING
 * You MUST check the return codes of ALL functions EVERY time they
 * are called, any one of them can fail. If you ignore the errors,
 * your file may become corrupt! This is nothing new.
 * WARNING WARNING WARNING WARNING WARNING WARNING WARNING WARNING
*/
#ifndef __FBUF_HEADER_INCLUDED__
#define __FBUF_HEADER_INCLUDED__

struct fbuf {
	int fd;
	char *buf; /* base of buffer */
	char *ptr; /* current pos in buffer */
	size_t buf_len; /* amount of buffer left */
	size_t buf_sz; /* total buffer size */
};

int fbuf_attach(struct fbuf *b, int fd, size_t bufsz);
int fbuf_flush(struct fbuf *b);
int fbuf_detach(struct fbuf *b);
int fbuf_close(struct fbuf *b);
int fbuf_newfd(struct fbuf *b, int fd);
int fbuf_write_slow(struct fbuf *b, void *buf, size_t len);

static inline int fbuf_write(struct fbuf *b, void *buf, size_t len)
{
	if ( len < b->buf_len ) {
		memcpy(b->ptr, buf, len);
		b->ptr += len;
		b->buf_len -= len;
		return 0;
	}

	return fbuf_write_slow(b, buf, len);
}

#endif /* __FBUF_HEADER_INCLUDED__ */
