/*
 * This file is part of tcpidx
 * Copyright (c) 2003 Gianni Tedesco
 * Released under the terms of the GNU GPL version 2
 *
 * Code for indexing ordered keys. This is the code for creating the
 * indexes in memory, you cannot query the values out of this code
 * (although the way it is architected, you could easily extend it to
 * do this). The internal memory structure is an AVL tree. When written
 * to disk the keys are written in order, then the values.
*/
#ifndef __I_ORDERED_HEADER_INCLUDED__
#define __I_ORDERED_HEADER_INCLUDED__

struct i_ordered {
	struct mpool avl_nodes;
	struct avl *cache;
	struct avl *root;
	size_t num_keys;
	size_t num_vals;
};

int i_ordered_new(struct i_ordered *db);
int i_ordered_write(struct i_ordered *db, struct fbuf *o);
int i_ordered_insert(struct i_ordered *db,
			idx_key_t key,
			idx_val_t val);
void i_ordered_free(struct i_ordered *db);
static inline size_t i_ordered_size(struct i_ordered *db)
{
	return (db->num_keys*sizeof(struct idx_node)) +
		(db->num_vals*sizeof(idx_val_t));
}
static inline size_t i_ordered_keys(struct i_ordered *db)
{
	return db->num_keys;
}
static inline size_t i_ordered_vals(struct i_ordered *db)
{
	return db->num_vals;
}

#endif /* __I_ORDERED_HEADER_INCLUDED__ */
